LivestockTrailerAnimation = {}

IDLE1 = "FA_idleLong01"
IDLE2 = "FA_idleLong02"
IDLE3 = "FA_idleLong03"

WALK_FRONT = "FA_walkFront01"
WALK_BACK = "FA_walkBack01"

MAX_SPEED_LIMIT = 6
MIN_TROT_SPEED = 5

function LivestockTrailerAnimation.prerequisitesPresent(specializations)
	return true
end

function LivestockTrailerAnimation.registerFunctions(vehicleType)
	SpecializationUtil.registerFunction(vehicleType, "startIdleAnimation", LivestockTrailerAnimation.startIdleAnimation)
	SpecializationUtil.registerFunction(vehicleType, "stopTimers", LivestockTrailerAnimation.stopTimers)
end

function LivestockTrailerAnimation.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", LivestockTrailerAnimation)
	SpecializationUtil.registerEventListener(vehicleType, "onUpdateTick", LivestockTrailerAnimation)
end

function LivestockTrailerAnimation.registerOverwrittenFunctions(vehicleType)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "onAnimalLoaded", LivestockTrailerAnimation.onAnimalLoaded)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "doCheckSpeedLimit", LivestockTrailerAnimation.doCheckSpeedLimit)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "updateAnimals", LivestockTrailerAnimation.updateAnimals)
end

function LivestockTrailerAnimation.initSpecialization()
	local schema = Vehicle.xmlSchema
    local baseKey = "vehicle.animations"

	schema:register(XMLValueType.BOOL, baseKey.."#shouldAnimalsWalkWhileDriving", "Should animals walk while driving", false)
end

function LivestockTrailerAnimation:onLoad(savegame)
	self.spec_livestockTrailerAnimation = {}
	local spec = self.spec_livestockTrailerAnimation
    spec.animalNodes = {}

    local baseKey = "vehicle.animations"

    spec.shouldAnimalsWalkWhileDriving = self.xmlFile:getValue(baseKey.."#shouldAnimalsWalkWhileDriving", false)
    spec.animalsAreWalking = false
    spec.animationTimer = {}

    if spec.shouldAnimalsWalkWhileDriving then
        self.speedLimit = MAX_SPEED_LIMIT
    end
end

function LivestockTrailerAnimation:updateAnimals(superFunc)
    self.spec_livestockTrailerAnimation.animalNodes = {}
    self:stopTimers()
    superFunc(self)
end

function LivestockTrailerAnimation:onAnimalLoaded(superFunc, i3dNode, failedReason, args)
    superFunc(self, i3dNode, failedReason, args)

	if i3dNode == 0 then
        return
	end

    local components = {}
    I3DUtil.loadI3DComponents(i3dNode, components)
    local skeletonNode = I3DUtil.indexToObject(components, "0")

    table.addElement(self.spec_livestockTrailerAnimation.animalNodes, skeletonNode)
    self:startIdleAnimation(skeletonNode)
end

function LivestockTrailerAnimation:startIdleAnimation(skeletonNode)
    local characterSet = getAnimCharacterSet(skeletonNode)

    disableAnimTrack(characterSet, 0)
    setAnimTrackTime(characterSet, 0, 0, true)
    clearAnimTrackClip(characterSet, 0)

    local idleName = LivestockTrailerAnimation.selectIdleAnimation()
    local clipIndex1 = getAnimClipIndex(characterSet, idleName)

    if clipIndex1 == nil then
        Logging.error("Clip "..idleName.." not exist for current animal type")
        return
    end

    assignAnimTrackClip(characterSet, 0, clipIndex1)
    setAnimTrackLoopState(characterSet, 0, true)
    setAnimTrackTime(characterSet, 0, 0, true)
    setAnimTrackSpeedScale(characterSet, 0, 1)
    setAnimTrackBlendWeight(characterSet, 0, 1)
    enableAnimTrack(characterSet, 0)

    local clipDuration = getAnimClipDuration(characterSet, clipIndex1)
    local timer = Timer.createOneshot(clipDuration, function()
        if self.spec_livestockTrailerAnimation.animalsAreWalking == false then
            self:startIdleAnimation(skeletonNode)
        end
    end)

    table.addElement(self.spec_livestockTrailerAnimation.animationTimer, timer)
end

function LivestockTrailerAnimation.selectIdleAnimation()
    local index = math.random(1,3)

    if index == 1 then
        return IDLE1
    elseif index == 2 then
        return IDLE2
    else
        return IDLE3
    end
end

function LivestockTrailerAnimation:doCheckSpeedLimit(superFunc)
    if self.spec_livestockTrailerAnimation.shouldAnimalsWalkWhileDriving and #self.spec_livestockTrailerAnimation.animalNodes > 0 then
        return true
    else
        return superFunc(self)
    end
end

function LivestockTrailerAnimation:onUpdateTick(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
    if #self.spec_livestockTrailerAnimation.animalNodes == 0 then
        return
    end

    local currentSpeed = self:getLastSpeed()

    if currentSpeed < 1 and self.spec_livestockTrailerAnimation.animalsAreWalking == true then
        self.spec_livestockTrailerAnimation.animalsAreWalking = false
        self.spec_livestockTrailerAnimation.lastSpeed = 0

        for index, skeletonNode in pairs(self.spec_livestockTrailerAnimation.animalNodes) do
            self:startIdleAnimation(skeletonNode)
        end
    elseif currentSpeed >= 1 and self.spec_livestockTrailerAnimation.animalsAreWalking == false and self.spec_livestockTrailerAnimation.shouldAnimalsWalkWhileDriving then
        self:stopTimers()

        for index, skeletonNode in pairs(self.spec_livestockTrailerAnimation.animalNodes) do
            local driveForward = self.lastSignedSpeedReal > 0
            LivestockTrailerAnimation.startWalkAnimation(skeletonNode, currentSpeed, driveForward)
        end

        self.spec_livestockTrailerAnimation.animalsAreWalking = true
        self.spec_livestockTrailerAnimation.lastSpeed = currentSpeed
    elseif currentSpeed >= 1 and self.spec_livestockTrailerAnimation.animalsAreWalking == true and self.spec_livestockTrailerAnimation.shouldAnimalsWalkWhileDriving then
        for index, skeletonNode in pairs(self.spec_livestockTrailerAnimation.animalNodes) do
            LivestockTrailerAnimation.updateWalkAnimation(skeletonNode, currentSpeed)
        end
    end
end

function LivestockTrailerAnimation.startWalkAnimation(skeletonNode, currentSpeed, driveForward)
    local characterSet = getAnimCharacterSet(skeletonNode)

    local walkClip = WALK_FRONT

    if driveForward == false then
        walkClip = WALK_BACK
    end

    setAnimTrackTime(characterSet, 0, 0, true)
    clearAnimTrackClip(characterSet, 0)

    local clipWalk = getAnimClipIndex(characterSet, walkClip)

    if clipWalk == nil then
        Logging.error("Walk clip "..walkClip.." not exist for current animal type")
        return
    end

    assignAnimTrackClip(characterSet, 0, clipWalk)
    setAnimTrackLoopState(characterSet, 0, true)
    setAnimTrackTime(characterSet, 0, 0, true)
    setAnimTrackSpeedScale(characterSet, 0, 0)
    setAnimTrackBlendWeight(characterSet, 0, 1)
    enableAnimTrack(characterSet, 0)
end

function LivestockTrailerAnimation.updateWalkAnimation(skeletonNode, currentSpeed)
    local characterSet = getAnimCharacterSet(skeletonNode)

    local minSpeed = 0
    local maxSpeed = MAX_SPEED_LIMIT
    local range = maxSpeed - minSpeed

    local factor = (currentSpeed - minSpeed) / range
    local scale = (factor * range) / maxSpeed

    setAnimTrackSpeedScale(characterSet, 0, scale)
end

function LivestockTrailerAnimation:stopTimers()
    for index, timer in pairs(self.spec_livestockTrailerAnimation.animationTimer) do
        timer:delete()
    end

    self.spec_livestockTrailerAnimation.animationTimer = {}
end